﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Traffic.Data.Models;
using Traffic.Data.ViewModels;
using Traffic.Repository.Infrastructures;
using Traffic.Service.Interfaces;

namespace Traffic.Service.Implements
{
    public class EventTypeService : IEventTypeService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly ILogger _logger;

        public EventTypeService(IUnitOfWork unitOfWork, ILogger<EventTypeService> logger)
        {
            _unitOfWork = unitOfWork;
            _logger = logger;
        }

        public IEnumerable<EventTypeViewModel> GetEventTypes()
        {
            try
            {
                var eventType = _unitOfWork.EventTypeRepository.GetEventTypes();
                var data = eventType.ToList().Select(x => new EventTypeViewModel
                {
                    Id = x.Id,
                    EventName = x.EventName,
                    EventType = x.EventType
                });

                return data;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
                return new List<EventTypeViewModel>();
            }
        }

        public IEnumerable<EventTypeViewModel> GetEventType1s()
        {
            try
            {
                var eventType = _unitOfWork.EventTypeRepository.GetEventType1s();
                var data = eventType.ToList().Select(x => new EventTypeViewModel
                {
                    Id = x.Id,
                    EventName = x.EventName,
                    EventType = x.EventType
                });

                return data;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
                return new List<EventTypeViewModel>();
            }
        }

        public PageViewModel<IEnumerable<EventTypeViewModel>> GetEventTypes(SearchModel searchModel)
        {
            try
            {
                var eventType = _unitOfWork.EventTypeRepository.GetEventTypes();

                if (!string.IsNullOrEmpty(searchModel.Term))
                {
                    eventType = eventType.Where(x => x.EventName.Contains(searchModel.Term) || x.EventType.Contains(searchModel.Term));
                }

                int count = eventType.Count();
                int pageCount = count % searchModel.PageSize != 0 ? (count / searchModel.PageSize) + 1 : count / searchModel.PageSize;

                switch (searchModel.Order.ToLower())
                {
                    case "id":
                    default:
                        eventType = searchModel.IsAsc ? eventType.OrderBy(x => x.Id) : eventType.OrderByDescending(x => x.Id);
                        break;
                    case "eventname":
                        eventType = searchModel.IsAsc ? eventType.OrderBy(x => x.EventName).ThenBy(x => x.Id) : eventType.OrderByDescending(x => x.EventName).ThenByDescending(x => x.Id);
                        break;
                    case "eventtype":
                        eventType = searchModel.IsAsc ? eventType.OrderBy(x => x.EventType).ThenBy(x => x.Id) : eventType.OrderByDescending(x => x.EventType).ThenByDescending(x => x.Id);
                        break;
                }

                eventType = eventType.Skip((searchModel.Page - 1) * searchModel.PageSize).Take(searchModel.PageSize);
                var data = eventType.ToList().Select(x => new EventTypeViewModel
                {
                    Id = x.Id,
                    EventName = x.EventName,
                    EventType = x.EventType
                });

                return new PageViewModel<IEnumerable<EventTypeViewModel>>
                {
                    RowCount = count,
                    PageCount = pageCount,
                    CurrentPage = searchModel.Page,
                    Data = data,
                };
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
                return new PageViewModel<IEnumerable<EventTypeViewModel>>();
            }
        }

        public EventTypeViewModel GetEventTypeById(int id)
        {
            try
            {
                var eventType = _unitOfWork.EventTypeRepository.GetEventTypeById(id);
                if (eventType != null)
                {
                    return new EventTypeViewModel
                    {
                        Id = id,
                        EventName = eventType.EventName,
                        EventType = eventType.EventType,
                    };
                }
                else
                {
                    return null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
                return null;
            }
        }

        public EventTypeViewModel GetEventTypeByEventType(string eventType)
        {
            try
            {
                var data = _unitOfWork.EventTypeRepository.GetEventTypeByEventType(eventType);
                if (data != null)
                {
                    return new EventTypeViewModel
                    {
                        EventName = data.EventName,
                        EventType = data.EventType,
                    };
                }
                else
                {
                    return null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
                return null;
            }
        }

        public TradeResultModel InsertEventType(EventTypeViewModel EventType)
        {
            try
            {
                var eventType = new EventTypeData
                {
                    EventName = EventType.EventName,
                    EventType = EventType.EventType,
                };

                var result = _unitOfWork.EventTypeRepository.InsertEventType(eventType);
                _unitOfWork.Complete();

                return new TradeResultModel { Success = result, Message = result ? "新增成功" : "新增失敗" };
            }
            catch (Exception ex)
            {
                _unitOfWork.Rollback();
                _logger.LogError(ex.Message);
                return new TradeResultModel() { Success = false, Message = ex.ToString() };
            }
        }

        public TradeResultModel UpdatEventType(int id, EventTypeViewModel EventType)
        {
            try
            {
                var eventType = new EventTypeData
                {
                    Id = id,
                    EventName = EventType.EventName,
                    EventType = EventType.EventType,
                };

                var result = _unitOfWork.EventTypeRepository.UpdateEventType(eventType);
                _unitOfWork.Complete();

                return new TradeResultModel { Success = result, Message = result ? "修改成功" : "修改失敗" };
            }
            catch (Exception ex)
            {
                _unitOfWork.Rollback();
                _logger.LogError(ex.Message);
                return new TradeResultModel() { Success = false, Message = ex.ToString() };
            }
        }

        public TradeResultModel DeleteEventType(int id)
        {
            try
            {
                var result = _unitOfWork.EventTypeRepository.DeleteEventType(id);
                _unitOfWork.Complete();

                return new TradeResultModel { Success = result, Message = result ? "刪除成功" : "刪除失敗" };
            }
            catch (Exception ex)
            {
                _unitOfWork.Rollback();
                _logger.LogError(ex.Message);
                return new TradeResultModel() { Success = false, Message = ex.ToString() };
            }
        }
    }
}
